@@ -5,7 +5,7 @@
 
 --------------------------------------------------------------------------------
 gSOAP XML Web services tools
-Copyright (C) 2001-2012, Robert van Engelen, Genivia Inc. All Rights Reserved.
+Copyright (C) 2000-2016, Robert van Engelen, Genivia Inc. All Rights Reserved.
 This software is released under one of the following licenses:
 GPL or Genivia's license for commercial use.
 --------------------------------------------------------------------------------
@@ -41,7 +41,7 @@ A commercial use license is available from Genivia, Inc., contact@genivia.com
 #include "httpda.h"
 #endif
 
-#define WSDL2H_VERSION "2.8.17r"
+#define WSDL2H_VERSION "2.8.35"
 
 #ifdef WIN32
 # pragma warning(disable : 4996)
@@ -94,6 +94,7 @@ extern int _flag,
            aflag,
            bflag,
 	   cflag,
+	   c11flag,
 	   dflag,
 	   eflag,
 	   fflag,
@@ -102,10 +103,12 @@ extern int _flag,
 	   jflag,
 	   kflag,
 	   mflag,
+	   Mflag,
 	   pflag,
 	   Pflag,
 	   Rflag,
 	   sflag,
+	   Uflag,
 	   uflag,
 	   vflag,
 	   wflag,
@@ -120,6 +123,7 @@ extern SetOfString exturis;
 
 #define MAXINFILES (1000)
 
+extern int openfiles;
 extern int infiles;
 extern char *infile[MAXINFILES], *outfile, *proxy_host, *proxy_userid, *proxy_passwd, *auth_userid, *auth_passwd;
 extern const char *mapfile, *import_path, *cwd_path, *cppnamespace;
@@ -132,9 +136,10 @@ extern const char *schema_prefix;
 extern const char elementformat[];
 extern const char pointerformat[];
 extern const char attributeformat[];
-extern const char vectorformat_open[];
-extern const char vectorformat[];
-extern const char pointervectorformat[];
+extern const char templateformat_open[];
+extern const char attrtemplateformat_open[];
+extern const char templateformat[];
+extern const char pointertemplateformat[];
 extern const char arrayformat[];
 extern const char arraysizeformat[];
 extern const char arrayoffsetformat[];
@@ -144,6 +149,9 @@ extern const char schemaformat[];
 extern const char serviceformat[];
 extern const char paraformat[];
 extern const char anonformat[];
+extern const char sizeparaformat[];
+extern const char pointertemplateparaformat[];
+
 extern const char copyrightnotice[];
 extern const char licensenotice[];
 
