@@ -273,8 +273,13 @@ int main(int argc, char **argv)
   fprintf(stderr, "[Note: http://127.0.0.1:%d for settings, login: '"AUTH_USERID"' and '"AUTH_PASSWD"']\n", port);
   fprintf(stderr, "[Note: you should enable Linux/Unix SIGPIPE handlers to avoid broken pipe]\n");
 
+  /* Init SSL (can skip or call multiple times, engien inits automatically) */
+  soap_ssl_init();
+  /* soap_ssl_noinit(); call this first if SSL is initialized elsewhere */
+
   soap_init2(&soap, SOAP_IO_KEEPALIVE, SOAP_IO_DEFAULT);
 
+  /* set up lSSL ocks */
   if (CRYPTO_thread_setup())
   {
     fprintf(stderr, "Cannot setup thread mutex\n");
@@ -360,10 +365,10 @@ void server_loop(struct soap *soap)
     SOAP_SOCKET sock;
     int newpoolsize;
     
-    soap->cookie_domain = options[OPTION_d].value;
-    soap->cookie_path = options[OPTION_p].value;
-    soap_set_cookie(soap, "visit", "true", NULL, NULL);
-    soap_set_cookie_expire(soap, "visit", 600, NULL, NULL);
+    soap->cookie_domain = options[OPTION_d].value; /* set domain of this server */
+    soap->cookie_path = options[OPTION_p].value;   /* set root path of the cookies */
+    soap_set_cookie(soap, "visit", "true", NULL, NULL); /* use global domain/path */
+    soap_set_cookie_expire(soap, "visit", 600, NULL, NULL); /* max-age is 600 seconds to expire */
 
     if (options[OPTION_c].selected)
       soap_set_omode(soap, SOAP_IO_CHUNK); /* use chunked HTTP content (fast) */
